---
id: motivation
title: Мотивация
---

![Глянцевая робофабрика](Images/robofactory_art.jpg)

Идея создания роботов, которые воспроизводят сами себя, занимает умы человечества с середины 20 века. Джон фон Нейман - отец-основатель информатики, был одним из первых, кто озадачился этим вопросом по серьёзному. Тем не менее, за прошедшие десятилетия, тема так и не вышла за рамки академических кругов. Все известные широкой публике [проекты](https://doi.org/10.1146/annurev-control-071819-010010) собирающих самих себя машин не нашли своего применения ни в промышленности, ни в повседневной жизни.

Вполне возможно, что это происходит по той причине, что самовоспроизводство не рассматривается с практической точки зрения - как технология, способная привнести ощутимый вклад улучшение качества жизни людей и общества. Эксперименты исследовательских ВУЗов лишь подкрепляют эту точку зрения.

В тоже время, коммерческие компании также не готовы прибегать к технологиями самовоспроизводства. Робототехнические комплексы не проектируются для автономной эксплуатации, а чрезмерная автоматизация несёт дополнительные риски. Развитие коллаборативных технологий ещё дальше смещает внимание компаний от такой постановки задачи.

То есть с одной стороны мы видим исключительно академические эксперименты, лишённые какой-либо прикладной ценности. С другой стороны мы наблюдаем за тем, как ведущие мировые компании избегают полностью автоматизировать производство.

В рамках проекта ___Robossembler___ мы постараемся разрешить это противоречие и использовать технологии самовоспроизводства для достижения практической пользы. На наш взгляд технологии самовоспроизводства действительно важны - они способны, например, обеспечить зачастую критически важную масштабируемость производственной системы (резко нарастить производство какого-то жизненно-важного для общества ресурса - например, респираторов).

## Предпосылки

Современные производственные системы (заводы, фабрики) представляют собой сложные программно-аппаратные комплексы, состоящие из большого количества компонентов с разной степенью взаимозависимости, которые разрабатываются, изготавливаются и поставляются разными компаниями. Глубокое разделение труда с одной стороны повышает качество, но с другой - затрудняет возможности интеграции компонентов в полностью автоматизированные надсистемы.
В свою очередь, задача разработки автономных (_light's out_) или самовоспроизводящихся (_self-replication_) технических систем требует тесной интеграции данных жизненного цикла в рамках общего для всех его подсистем требования «безлюдности». Данное требование также налагает ряд существенных ограничений на дизайн всех компонентов системы и делает почти невозможным разработку и внедрение в рамках _отдельно взятой компании_. Даже наиболее автоматизированные отрасли промышленности (например, автомобильная) не могут уйти от ручного труда; в особенности на сборочных операциях.

Однако полностью автоматизированное производство обладает и рядом преимуществ, потому как позволяет достичь высокого синергетического эффекта за счёт отказа от
* человеко-машинных интерфейсов и требований к эстетичности (в том числе специальная маркировка);
* условий эксплуатации, адаптированных под человека (воздух, влажность, освещение, температура);
* соблюдения стандартов безопасности, взаимозаменяемости в рамках существующей номенклатуры компонентов, ремонтопригодности.

То есть разработка подобного рода систем требует пересмотра многих компонентов технических систем и принципов производственного процесса, который в настоящее время ориентирован под сборку, наладку и эксплуатацию человеком.

Однако то, что невозможно для отдельно взятой компании, может быть возможным для международного движения open source. Открытость процесса разработки и документации посодействует непрерывной интеграции данных жизненного цикла подсистем, разрабатываемых отдельными командами, что затруднительно в условиях закрытых инжиниринговых фирм.

Также, современный уровень вычислительных систем на текущем этапе их развития делает возможным создание цифровых двойников (_digital twin_) производств, поведение которых в рамках симуляции может быть приближено к реальным физическим объектам. В перспективе станет возможным не дожидаться опытного производства, чтобы проверить какую-то гипотезу, а ограничиться физическим и имитационным моделированием. Разработчики не будут скованы ограничениями существующей компонентной базы и обусловленной этим инерцией мышления. Допустимо придумать всё с нуля — с учётом требования полной автоматизированности.

## Робосборщик

Робосборщик выстраивает мост между инженерами, потребителями и производителями, позволяя добиться оптимального результата. Инженер может проектировать, исходя из возможности автоматизированного производства, а потребитель может выбрать какой вариант может быть для него более оптимальным - дешёвое и простое или дорогое и красивое.

Архитектура разрабатываемых в Robossembler решений позволит развернуть производственную инфраструктуру даже небольшим организациям - FabLab'ам.

Robossembler - это проект для того, чтобы сделать программирование роботов более доступным, благодаря технологиям автоматизированного проектирования и внедрения:
* Обучение с подкреплением в виртуальных средах
* Sim2Real transfer - перенос программ, полученных из симуляции, в реальный мир
* Поиск оптимальных последовательностей сборки - Assembly Sequence Planning
* Планирование производства

Использование решений с открытым исходным кодом снижает затраты на разработку приложений для робототехники. Массовая робототехника - значит дешёвая робототехника. Open Source & Robossembler - быстрая и доступная разработка программ для управления доступными и потому массовыми роботами.

Ориентируясь на те технологии, которые позволяют автономно (без участия человека) производить нужные товары, мы можем применять их и для производства самих производственных систем. Например, можно спроектировать робота-манипулятора так, чтобы его было удобно собирать другими роботами-манипуляторами.

## Потребительские стимулы к автоматизации через развитие кооперации

Компании в поисках способов повышения производительности через автоматизацию, как правило, предпочитают усложнять самих роботов и их программы. С точки зрения _системной инженерии_ они меняют __обеспечивающую__ систему, потому что не могут пожертвовать потребительскими качествами своей продукции и проиграть конкуренцию на рынке. Но есть и другой путь - изменить качества продукции для удобства их автоматизированного изготовления (то есть изменить саму __целевую__ систему или продукт) в допустимых пределах и дать возможность потенциальным потребителям выбор.

По сей день в автоматизации производства были заинтересованы прежде всего производители. Для них автоматизация - это способ избавиться от человеческого фактора, повысить предсказуемость и мощность производства. В тоже время покупатель делает выбор, руководствуясь ценой, оставляя сам технологический процесс за рамками своего рассмотрения.

Если же покупатель и владелец производства являются одним и тем же лицом (или группой лиц - как, например, в производственно-потребительском кооперативе), то ситуация меняется:
* потребитель начинает принимать решение не в тот момент, когда произведённый товар оказывается на прилавке, а раньше - вплоть до момента его замысливания. Он получает возможность оказать влияние на продукт на каждой фазе его жизненного цикла. Развитие технологий массовой координации людей через сеть интернет(краудфандинг, краудвестинг; т.н. викиномика) даёт нам основания полагать, что эта тенденция имеет место быть - потребители всё чаще вовлекаются на ранних фазах развития продукта. Коллективное владение обеспечивающей системой может усилить мотивацию к данному поведению и снизить риски по выводу продукта на рынок, переводя конкуренцию в мир презентаций и моделей
* потребитель осведомлён об архитектуре производственной системы, которой он совладеет, и принимает решение не только исходя потребительских качеств продукта, но и с учётом других факторов: экологичности и стоимости производства, оптимальности логистики
* потребитель может координировать своё поведение с другими членами организации совладельцев, организованно управляя спросом и экономикой в целом.

Проект Робосборщик предлагает проектировать и внедрять сверх-автоматизированные производства через развитие кооперации.